/* ez-elf.h: elf(64) processing

   Copyright (C) 2019 Michael Zucchi

   This program is free software: you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation, either version 3 of
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this program. If not, see
   <http://www.gnu.org/licenses/>.
*/

#ifndef EZ_ELF_H
#define EZ_ELF_H

#include <elf.h>

typedef struct ez_elf ez_elf;

typedef Elf64_Xword elf_addr_t;

struct ez_elf *ez_elf_open(const char *name);
void ez_elf_close(struct ez_elf *elf);
elf_addr_t ez_elf_addr(struct ez_elf *elf, const char *name, int *size, const void **ptr);
const char *ez_elf_name(struct ez_elf *elf, elf_addr_t addr, int *size);

/**
 * Read an address pointer from a structure inside the elf file.
 * If the address has a relocation entry then it is relocated.
 */
const void *ez_elf_read_addr(struct ez_elf *elf, const void *addr);

#endif
