/* ez-port.h: A simple inter-thread IPC mechanism with poll() support.

   Copyright (C) 2019 Michael Zucchi

   This program is free software: you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation, either version 3 of
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this program. If not, see
   <http://www.gnu.org/licenses/>.
*/

#ifndef _EZ_PORT_H
#define _EZ_PORT_H

/**
 * This is trivial multiple-writer / multiple-reader non-copying and
 * blocking message queue with support for poll() and select().
 *
 * It can only be used for in-process message passing, i.e. between
 * threads.
 */

typedef struct ez_port ez_port;

/**
 * Create a new port.
 *
 * @param limit maximum number of pending messages.
 * @return the new port, or NULL if it cannot be created.  errno will
 * indicate the error.
 */
ez_port *ez_port_new(int limit);

/**
 * Free a port.
 */
void ez_port_free(ez_port *port);

/**
 * Retrieve a file descriptor that can be polled on.
 */
int ez_port_fd(ez_port *port);

/**
 * Put a new message onto the port.  NULL is allowed.
 *
 * If the queue is full then this call will block.
 *
 * @return 0 on success, on failure errno indicates error.
 */
int ez_port_put(ez_port *port, void *msg);

/**
 * Retrieve the next message from the port if one is available.
 */
void *ez_port_poll(ez_port *port);

/**
 * Take the next message from the port, waiting if none are available.
 *
 * @return the next message, or NULL if none are available (or an error occured).
 */
void *ez_port_take(ez_port *port);

#endif
